:-module buttonsWAction.
/*  NOTICE:
    This module needs an extended version of mtk.m, as explained by the
	Linux Magazine example - it will not work with the library provided
	by the extras distribution of Mercury 0.13.1.
*/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:-interface.
:-import_module io.

:-pred main(io::di, io::uo) is det.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:-implementation.
:-import_module string.  %% for '++'
:-import_module list.
:-import_module require.  %%  for error/1
:-import_module mtcltk, mtk.  %%  Tcl/Tk modules

/*  The main predicate:
*/
main(!IO) :-
	mtcltk.main(
		pred(Tk::in, I::di, O::uo) is det :-
			task(Tk, I, O),
			["Buttons"],
			!IO).

/*  The application behaviour tp perform - it is inserted as a
	higher order argument into mtcltk.main/4:
*/
:-pred task(tcl_interp, io, io).
:-mode task(in, di, uo) is det.
task(Tk, !IO) :-
	Frame = mtk.root_window,
	%%  configure the frame:
	configure(Tk, Frame,
			  %%  list of frame configs:
			  [height(40), width(400), padx(50), pady(10),
			   background("green")],
			  !IO),
	%%  provide the button:
	createButton(":-0", "yellow", Tk, Frame, YellowButton, !IO),
	createButton(":-o", "red", Tk, Frame, RedButton, !IO),
	createButton(";-)", "blue", Tk, Frame, BlueButton, !IO),
	%%  place the buttons inside the frame:
	mtk.pack(Tk,
			 [pack(YellowButton, []),
			  pack(RedButton, []),
			  pack(BlueButton, [])],
			 !IO).


/*  Creates a button of given specification:
*/
:-pred createButton(string, string, tcl_interp, widget, widget, io, io).
:-mode createButton(in, in, in, in(toplevel), out, di, uo) is det.
createButton(Label, Color, Tk, Frame, Button, !IO) :-
	mtk.button(Tk,
			   %%  list of button configs:
			   [text(Label), padx(50),
				background(Color), active_background(Color)],
			   Frame,
			   Button,
			   !IO),
	configure(Tk, Button, [command(reportColor(Button))], !IO).


/*  Action to be executed when clicking a button.
	Uses 'cget_string' to read a string configuration from Tk
	for simple output to STDOUT.
	It is inserted as higher order argument as a button configuration.
*/
:-pred reportColor(widget, tcl_interp, io, io).
:-mode reportColor(in(button), in, di, uo) is det.
reportColor(Button, Tk, !IO) :-
	cget_string(Tk, Button, "background", BgColor, !IO),
	io.write_string("Clicked: '-background "++BgColor++"'", !IO),
	io.nl(!IO).

